import { system,ItemStack,world,EquipmentSlot,EntityComponentTypes } from "@minecraft/server";
import { RawtextBuilder, showGuidePage, showPage } from "./GuideBook";
import { ModalFormData, ActionFormData } from "@minecraft/server-ui";

export const VERSION = "1.1.0";
export const VERSION_HISTORY = ["1.1.0"];

const settingsOrder = [
  "torchBrightness",
  "lanternBrightness",
  "blockBrightness",
];

const defaultValues = {
  torchBrightness: 6,
  lanternBrightness: 12,
  blockBrightness: 8,
};

  if (world.scoreboard.getObjective("settings") == undefined) {
    world.scoreboard.addObjective("settings", "settings");
  }

  settingsOrder.forEach((setting) => {
    const objective = world.scoreboard.getObjective("settings");
    try {
      objective.getScore(setting);
    } catch (error) {
      // Set the default value if an error occurs
      const defaultValue = defaultValues[setting] || 0; // Use 0 if not specified
      objective.setScore(setting, defaultValue);
    }
  });

function removeSingleItem(item, player) {
  const equippable = player.getComponent("equippable");
  const mainHandItem = equippable.getEquipment(EquipmentSlot.Mainhand);

  if (mainHandItem && mainHandItem.typeId === item.typeId) {
    if ((item.amount == 1)) {
      equippable.setEquipment(
        EquipmentSlot.Mainhand,
        new ItemStack("minecraft:air")
      );
    } else {
      const newItemStack = new ItemStack(item.typeId, mainHandItem.amount - 1);
      equippable.setEquipment(EquipmentSlot.Mainhand, newItemStack);
    }
  }
}

system.runInterval(() => {
  for (const player of world.getAllPlayers()) {
    const inventory = player.getComponent(EntityComponentTypes.Inventory);

    if (inventory === undefined) return;
    const container = inventory.container;
    if (container === undefined) return;

    // Update inventory items with the typeId "sg_tpa:guide"
    for (let i = 0; i < container.size; i++) {
      const itemStack = container.getItem(i);
      if (itemStack === undefined) continue;
      if (itemStack.typeId === "sg_light:guide") {
        const lore = itemStack.getLore() || []; // Get the existing lore or start with an empty array
        if (!lore.includes("§r§7Bedrock Essential")) {
          lore.push("§r§7Bedrock Essential");
          itemStack.setLore(lore);
          container.setItem(i, itemStack);
        }
      }
    }
  }
});

const lightSources = {
  torchLight: [
    "minecraft:torch",
    "minecraft:redstone_torch",
    "minecraft:soul_torch",
    "minecraft:sea_pickle",
    "minecraft:glow_berries",
    "sg_light:guide",
  ],
  lanternLight: [
    "minecraft:lantern",
    "minecraft:soul_lantern",
    "minecraft:lava_bucket",
  ],
  blockLight: [
    "minecraft:beacon",
    "minecraft:glowstone",
    "minecraft:lit_pumpkin",
    "minecraft:sea_lantern",
    "minecraft:end_rod",
    "minecraft:magma",
    "minecraft:ochre_froglight",
    "minecraft:pearlescent_froglight",
    "minecraft:verdant_froglight",
  ],
};

// Track the last type of light source the player was holding
const playerLastLightType = new Map();

// Function to clear light blocks around the player
function removeLight(player) {
  for (let i = 0; i <= 15; i++) {
    player.runCommand(`fill ~-1 ~-1 ~-1 ~1 ~1 ~1 minecraft:air replace minecraft:light_block_${i}`);
  }
}

// Function to create a 4-block thick solid outline to clear light blocks
function clearOuterCube(player) {
  for (let i = 0; i <= 15; i++) {
    // Create a solid 4-block thick cube around the player's light area
    player.runCommand(`fill ~-4 ~-4 ~-4 ~4 ~-2 ~4 minecraft:air replace minecraft:light_block_${i}`);
    player.runCommand(`fill ~-4 ~4 ~-4 ~4 ~2 ~4 minecraft:air replace minecraft:light_block_${i}`);
    player.runCommand(`fill ~-4 ~-3 ~-4 ~-2 ~3 ~4 minecraft:air replace minecraft:light_block_${i}`);
    player.runCommand(`fill ~4 ~-3 ~-4 ~2 ~3 ~4 minecraft:air replace minecraft:light_block_${i}`);
    player.runCommand(`fill ~-3 ~-3 ~-4 ~3 ~3 ~-2 minecraft:air replace minecraft:light_block_${i}`);
    player.runCommand(`fill ~-3 ~-3 ~4 ~3 ~3 ~2 minecraft:air replace minecraft:light_block_${i}`);
  }
}

// Main system run interval to handle dynamic light emission
system.runInterval(() => {
  for (const player of world.getAllPlayers()) {
    const equippable = player.getComponent("equippable");
    const mainHandItem = equippable.getEquipment(EquipmentSlot.Mainhand);
    const objective = world.scoreboard.getObjective("settings");

    // Determine current light source type and intensity for mainhand
    let currentLightType = null;
    let mainHandIntensity = 0;

    if (lightSources.torchLight.includes(mainHandItem?.typeId)) {
      currentLightType = "torch";
      mainHandIntensity = objective.getScore("torchBrightness");
    } else if (lightSources.lanternLight.includes(mainHandItem?.typeId)) {
      currentLightType = "lantern";
      mainHandIntensity = objective.getScore("lanternBrightness");
    } else if (lightSources.blockLight.includes(mainHandItem?.typeId)) {
      currentLightType = "block";
      mainHandIntensity = objective.getScore("blockBrightness");
    }

    // Check if mainHandItem is undefined (empty hand) and handle accordingly
    if (!mainHandItem) {
      if (player.hasTag("sg_light:emitting")) {
        removeLight(player); // Clear the outer shell to prevent leftover light blocks
        player.removeTag("sg_light:emitting");
        playerLastLightType.delete(player.id); // Clear the last light type for the player
      }
      continue; // Skip further checks for an empty hand
    }

    // Get the last type of light source the player was holding
    const lastLightType = playerLastLightType.get(player.id);

    // Check if the player is holding a light source and run the corresponding command
    if (currentLightType) {
      if (lastLightType && lastLightType !== currentLightType) {
        // If switching to a different light source, clear the old light blocks
        removeLight(player);
      }
      
      clearOuterCube(player); // Clear the outer shell to prevent leftover light blocks
      
      player.runCommand(`fill ~-1 ~-1 ~-1 ~1 ~1 ~1 minecraft:light_block_${mainHandIntensity} replace air`);
      if (!player.hasTag("sg_light:emitting")) player.addTag("sg_light:emitting");
      
      // Update the last light type for the player
      playerLastLightType.set(player.id, currentLightType);
    } else {
      // If the player is not holding a light-emitting item, remove the light blocks once
      if (player.hasTag("sg_light:emitting")) {
        removeLight(player);
        player.removeTag("sg_light:emitting");
        playerLastLightType.delete(player.id); // Clear the last light type for the player
      }
    }
  }
});









world.afterEvents.playerSpawn.subscribe((event) => {
  const { initialSpawn, player } = event;
  if (!initialSpawn) return;

  const version = player.getDynamicProperty("sg_light_version");
  if (version === undefined) {
    player.setDynamicProperty("sg_light_version", VERSION);
    if (player.hasTag("sg_light_received_guide")) {
      player.removeTag("sg_light_received_guide");
      player.sendMessage({
        translate: "sg_light.announce_update",
        with: [VERSION],
      });
      return;
    }
    player.dimension.spawnItem(
      new ItemStack("sg_light:guide"),
      player.location
    );
  } else if (version !== VERSION) {
    player.setDynamicProperty("sg_light_version", VERSION);
    player.sendMessage({
      translate: "sg_light.announce_update",
      with: [VERSION],
    });
  }
});



const BUTTONS = [
  {
    // OPTIONS
    translation_key: "sg_light.guide.options.title",
    icon_path: "textures/ui/book_edit_default.png",
    action: (player) => {

      const objective = world.scoreboard.getObjective("settings");

      // Create and show the modal form
      const form = new ModalFormData()
        .title("Options")
        .slider(
          "Torch Brightness",
          1,
          15,
          1,
          objective.getScore("torchBrightness")
        )
        .slider(
          "Lantern Brightness",
          1,
          15,
          1,
          objective.getScore("lanternBrightness")
        )
        .slider(
          "Held Block Brightness",
          1,
          15,
          1,
          objective.getScore("blockBrightness")
        )

      form.show(player).then((res) => {
        if (res.canceled) return;

        // Iterate over form values and map them to the settings labels
        const settings = res.formValues;
        settings.forEach((value, index) => {
          const label = settingsOrder[index];
          if (label) {
            objective.setScore(label, value);
          }
        });
      });
    },
  },
  {
    // CHANGELOG
    translation_key: "sg_light.guide.changelog.title",
    icon_path: "textures/ui/book_addtextpage_default.png",
    action: (player) => {
      const body = new RawtextBuilder();
      body.pushLine({
        translate: "sg_light.guide.changelog.body",
        with: [VERSION],
      });
      body.pushLine();
      for (const version of VERSION_HISTORY) {
        body.pushLine({
          translate: "sg_light.guide.changelog.subtitle",
          with: [version],
        });
        body.pushLine();
        body.pushLine({
          translate: `sg_light.guide.changelog.${version}`,
          with: ["\n"],
        });
        body.pushLine();
        body.pushLine();
      }
      showPage(player, "sg_light.guide.changelog.title", body.build(), [
        {
          translation_key: "sg_light.guide.common.back_button",
          action: (player) => showMainPage(player),
        },
      ]);
    },
  },
  {
    translation_key: "sg_light.guide.common.exit_button",
  },
];

const showMainPage = (player) => {
  showGuidePage(player, "main", 4, BUTTONS);
};

world.afterEvents.itemUse.subscribe((event) => {
  system.run(() => {
    if (event.itemStack.typeId === "sg_light:guide") {
      showMainPage(event.source);
    }
  });
});